# scatterminal
ターミナル上に散布図を描画します

## 必要要件
- Python >= 3.10

## インストール
```shell
pip install scatterminal
```
このとき、`plot` というコマンドが利用できるようになります。

## 使い方
このツールには以下の２種類のデータソースを利用できます
### 1. CSV ファイル
ターミナル上で次のようにコマンドを実行するとcsvファイル（あるいはtsvファイル）を読み込んで散布図が描画されます。
```shell
plot tests/samples/single_column.csv
```
```
  12.000+
        |                                        *  *   *
  11.000+                                     *
  10.000+                                                  *
        |                                  *                  *
v 9.000 +                                                        *
a 8.000 +
l 7.000 +                              *                             *
u       |
e 6.000 +        *                  *
  5.000 +     *     *  *
        |                 *   *  *
  4.000 +----+--------------+---------------+--------------+--------------+
           0.000          5.000          10.000         15.000       20.000


*: value
```

主にターミナル上での利用を想定していますが、Python スクリプト内でも実行でき、同じ結果を得られます。
```python
from scatterminal.plot import plot_csv

plot_csv(["tests/samples/single_column.csv"])
```
### 2. Python の `list` オブジェクト
こちらは Python スクリプトでのみ利用可能です。

```python
from scatterminal.plot import plot_inline
from scatterminal.data_layer_model import SimpleDataSequence

x_arr = [x - 2 for x in range(8)]
y_arr1 = [x * x/2 - 2 * x for x in x_arr]
y_arr2 = [-x * x/2 + 1 * x + 3 for x in x_arr]
plot_inline(
    [
        SimpleDataSequence(x_arr, y_arr1, "y1"),
        SimpleDataSequence(x_arr, y_arr2, "y2"),
    ]
)
```
```
7.500  +
       |     *
5.000  +
       |                             o
       |             *       o              o                       *
2.500  +             o                              o
       |
0.000  +                     *                              *
       |     o                       *              *       o
-2.500 +                                    *
       |
-5.000 +                                                            o
       |-+-------------------+------------------+-------------------+------
      -2.500               0.000              2.500               5.000


*: y1  o: y2
```

## 細かな仕様
### データ系列
#### 基本形式
描画できるファイル形式の内、最も基本的な形式が次の例です。
それぞれの列はある一つのデータ系列のx軸座標、y軸座標を表します。
```csv:test/samples/double_column.csv
x,y
0.0000,0.0000
0.6283,0.5878
1.2566,0.9511
1.8850,0.9511
2.5133,0.5878
3.1416,0.0000
3.7699,-0.5878
4.3982,-0.9511
5.0265,-0.9511
5.6549,-0.5878
6.2832,-0.0000
6.9115,0.5878
```
#### 省略された形式
列が１本の場合はx軸座標が省略されたものと解釈されます。
```csv:tests/samples/single_column.csv
value
5.5
6.0
5.2
5.2
4.8
4.6
4.9
5.7
7.6
9.4
10.4
11.3
11.4
11.3
10.3
9.7
8.4
7.0
```
省略された x軸座標は暗黙的に0始まりの整数列で補われます。従って上の例は次の２列の形式と等価です。
```csv
,value
0,5.5
1,6.0
2,5.2
3,5.2
4,4.8
5,4.6
6,4.9
7,5.7
8,7.6
9,9.4
10,10.4
11,11.3
12,11.4
13,11.3
14,10.3
15,9.7
16,8.4
17,7.0
```
#### 複数のデータ系列
複数のデータ系列を描画する場合は複数のファイルを同時に読み込みます。以下の例では、2つの系列の各座標（つまり、`x1`、`y1`、`x2`及び`y2`）が必要になります。
```shell
plot tests/samples/single_column.csv tests/samples/single_column_seq2.csv
```
```
       |
       |                                         o  o  o
       |                                     o             o
12.500 +                                  o                   o
       |                                         *  *  *         o
10.000 +                               o     *             *        o
       |                                  *                   *
7.500  +     o  o  o  o             o                            *
       |                  o  o  o      *
       |                                                            *
5.000  +     *  *  *  *             *
       |                  *  *  *
2.500  +----+---------------+--------------+---------------+--------------+
          0.000           5.000         10.000          15.000       20.000


*: value  o: value2
```
しかし、時には複数のデータ系列が1つのx軸座標を共有する場合があります。この場合はファイルを1つにまとめることが可能で、各列は `x`、`y1`、`y2` を表します。
```csv:tests/samples/triple_column.csv
x,sin,cos
0.0000,0.0000,1.0000
0.6283,0.5878,0.8090
1.2566,0.9511,0.3090
1.8850,0.9511,-0.3090
2.5133,0.5878,-0.8090
3.1416,0.0000,-1.0000
3.7699,-0.5878,-0.8090
4.3982,-0.9511,-0.3090
5.0265,-0.9511,0.3090
5.6549,-0.5878,0.8090
6.2832,-0.0000,1.0000
6.9115,0.5878,0.8090
```
これを描画すると次のようになります。
```shell
plot tests/samples/triple_column.csv
```
```
       |
1.000  +     o         *    *                                  o
       |          o                                       o         o
       |          *              *                                  *
0.500  +               o                             o
       |
0.000  +     *                        *                        *
       |
-0.500 +                    o                   o
       |                                   *              *
       |                         o         o
-1.000 +                              o         *    *
       |-----+-------------------+-------------------+-------------------+-
           0.000               2.500               5.000              7.500
                                         x

*: sin  o: cos
```

### 対数
`--yscale log` のオプションを使用すれば対数表示になります。
```shell
plot tests/samples/double_column_power.csv tests/samples/double_column_power_seq2.csv --yscale log
```
```
        |                                                            o
316.228 +                                       *  *              o
        |                                     *              o  o
100.000 +                                   *           o o
        |                      o  o o o  o  o o o  o  o
31.623  +                           *
        |                      *  *
10.000  +                   *
        |                 *
3.162   +            *  *
        |          *
        |       *
1.000   +----+*----------+------------+-----------+-----------+-----------+
           0.000       5.000       10.000      15.000      20.000    25.000


*: power_val  o: power_val2
```